"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.postgraphile = exports.makeSchema = exports.makePgSources = void 0;
// Copy the types through for our dependents
require("graphile-build");
require("graphile-build-pg");
const grafast_1 = require("grafast");
const graphile_config_1 = require("graphile-config");
const schema_js_1 = require("./schema.js");
var schema_js_2 = require("./schema.js");
Object.defineProperty(exports, "makePgSources", { enumerable: true, get: function () { return schema_js_2.makePgSources; } });
Object.defineProperty(exports, "makeSchema", { enumerable: true, get: function () { return schema_js_2.makeSchema; } });
function postgraphile(preset) {
    const config = (0, graphile_config_1.resolvePresets)([preset]);
    let serverParams;
    let stopWatchingPromise = null;
    if (config.server?.watch) {
        serverParams = (0, grafast_1.defer)();
        stopWatchingPromise = (0, schema_js_1.watchSchema)(preset, (error, newParams) => {
            if (error || !newParams) {
                console.error("Watch error: ", error);
                return;
            }
            const oldServerParams = serverParams;
            serverParams = newParams;
            if (oldServerParams !== null &&
                "resolve" in oldServerParams &&
                typeof oldServerParams.resolve === "function") {
                oldServerParams.resolve(serverParams);
            }
            if (server) {
                server.setParams(serverParams);
            }
        });
    }
    else {
        serverParams = (0, schema_js_1.makeSchema)(preset);
    }
    let server;
    let released = false;
    function assertAlive() {
        if (released) {
            throw new Error(`PostGraphile instance has been released`);
        }
    }
    return {
        async getGrafserv() {
            assertAlive();
            if (!server) {
                const { grafserv } = await import("grafserv");
                server = grafserv(config, serverParams);
                server.onRelease(() => {
                    if (!released) {
                        throw new Error(`Grafserv instance released before PostGraphile instance; this is forbidden.`);
                    }
                });
            }
            return server;
        },
        async getServerParams() {
            assertAlive();
            return serverParams;
        },
        async getSchema() {
            assertAlive();
            return (await serverParams).schema;
        },
        async release() {
            assertAlive();
            released = true;
            if (server) {
                await server.release();
            }
            if (stopWatchingPromise) {
                try {
                    const cb = await stopWatchingPromise;
                    cb();
                }
                catch (e) {
                    /* nom nom nom */
                }
            }
        },
    };
}
exports.postgraphile = postgraphile;
exports.default = postgraphile;
//# sourceMappingURL=index.js.map