"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeV4Preset = void 0;
require("graphile-config");
const graphile_build_pg_1 = require("graphile-build-pg");
const PgV4BehaviorPlugin_js_1 = require("../plugins/PgV4BehaviorPlugin.js");
const PgV4InflectionPlugin_js_1 = require("../plugins/PgV4InflectionPlugin.js");
const PgV4NoIgnoreIndexes_js_1 = require("../plugins/PgV4NoIgnoreIndexes.js");
const PgV4SmartTagsPlugin_js_1 = require("../plugins/PgV4SmartTagsPlugin.js");
function isNotNullish(arg) {
    return arg != null;
}
const makeV4Plugin = (options) => {
    const { classicIds = false } = options;
    return {
        name: "PostGraphileV4CompatibilityPlugin",
        version: "0.0.0",
        inflection: {
            replace: {
                ...(classicIds
                    ? null
                    : {
                        // Rename GraphQL Global Object Identification 'id' to 'nodeId'
                        // TODO: this will be better as `_id` in general, but V4 uses `nodeId`
                        nodeIdFieldName() {
                            return "nodeId";
                        },
                        // Don't rename 'id' to 'rowId'
                        column(previous, options, details) {
                            const columnFieldName = this.camelCase(this._columnName(details));
                            return columnFieldName;
                        },
                    }),
            },
        },
        schema: {
            hooks: {
                build: {
                    callback(build) {
                        switch (options.simpleCollections) {
                            case "both": {
                                build.behavior.addDefaultBehavior("+connection +list");
                                break;
                            }
                            case "only": {
                                build.behavior.addDefaultBehavior("-connection +list");
                                break;
                            }
                            case "omit": {
                                build.behavior.addDefaultBehavior("+connection -list");
                                break;
                            }
                        }
                        // We could base this on the legacy relations setting; but how to set deprecated?
                        build.behavior.addDefaultBehavior("-singularRelation:connection -singularRelation:list");
                        return build;
                    },
                },
            },
        },
    };
};
function parseJWTType(type) {
    const parts = type.split(".");
    // TODO: parse this better!
    if (parts.length !== 2) {
        throw new Error("Cannot parse JWT type - it must have schema and type name separated by a period");
    }
    return parts.map((part) => {
        if (part[0] === '"') {
            if (part[part.length - 1] !== '"') {
                throw new Error(`Cannot parse JWT type; invalid quoting '${part}'`);
            }
            return part.slice(1, part.length - 1);
        }
        else {
            return part;
        }
    });
}
const makeV4Preset = (options = {}) => {
    const { pgUseCustomNetworkScalars, ...otherGraphileBuildOptions } = options.graphileBuildOptions ?? {};
    return {
        plugins: [
            ...(options.ignoreRBAC === false ? [graphile_build_pg_1.PgRBACPlugin] : []),
            PgV4InflectionPlugin_js_1.PgV4InflectionPlugin,
            PgV4SmartTagsPlugin_js_1.PgV4SmartTagsPlugin,
            PgV4BehaviorPlugin_js_1.PgV4BehaviorPlugin,
            ...(options.ignoreIndexes === false ? [PgV4NoIgnoreIndexes_js_1.PgV4NoIgnoreIndexesPlugin] : []),
            makeV4Plugin(options),
            ...(options.appendPlugins ? options.appendPlugins : []),
        ].filter(isNotNullish),
        disablePlugins: [
            ...(options.disableDefaultMutations
                ? ["PgMutationCreatePlugin", "PgMutationUpdateDeletePlugin"]
                : []),
            ...(options.skipPlugins ? options.skipPlugins.map((p) => p.name) : []),
        ],
        schema: {
            ...otherGraphileBuildOptions,
            pgUseCustomNetworkScalars: pgUseCustomNetworkScalars ?? false,
            pgV4UseTableNameForNodeIdentifier: true,
            pgForbidSetofFunctionsToReturnNull: options.setofFunctionsContainNulls === false,
            jsonScalarAsString: options.dynamicJson !== true,
            ...(options.jwtSecret != null
                ? {
                    pgJwtSecret: options.jwtSecret,
                }
                : null),
        },
        gather: {
            ...(options.jwtPgTypeIdentifier
                ? {
                    pgJwtType: parseJWTType(options.jwtPgTypeIdentifier),
                }
                : null),
        },
        grafast: {
            ...(options.additionalGraphQLContextFromRequest || options.pgSettings
                ? {
                    async context(ctx) {
                        const context = Object.create(null);
                        if (options.additionalGraphQLContextFromRequest) {
                            const addl = await options.additionalGraphQLContextFromRequest(ctx.httpRequest, ctx.httpRequest?.res);
                            Object.assign(context, addl);
                        }
                        if (options.pgSettings) {
                            const pgSettings = typeof options.pgSettings === "function"
                                ? await options.pgSettings(ctx.httpRequest)
                                : options.pgSettings;
                            Object.assign(context, { pgSettings });
                        }
                        return context;
                    },
                }
                : null),
        },
    };
};
exports.makeV4Preset = makeV4Preset;
exports.default = (0, exports.makeV4Preset)();
//# sourceMappingURL=v4.js.map